package com.icomsys;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.icomsys.dsl.MessageSendServiceDsl;
import com.icomsys.util.CommonBeanUtils;
import com.icomsys.vo.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

public class MessagingConnector {

    private final String sms;
    private final String lms;
    private final String mms;
    private final String kakao;
    private final String apiKey;
    private final MessageSendServiceDsl messageSendServiceDsl;
    private final HttpClient httpClient;

    public MessagingConnector(String sms, String lms, String mms, String kakao, String apiKey, MessageSendServiceDsl messageSendServiceDsl) {
        this.sms = sms;
        this.lms = lms;
        this.mms = mms;
        this.kakao = kakao;
        this.apiKey = apiKey;
        this.httpClient = HttpClient.newHttpClient();
        this.messageSendServiceDsl = messageSendServiceDsl;
    }

    public MessageSendResponse messageDiv(MessageSendRequest messageSendRequest) throws IOException, InterruptedException {
        String url = sms;
        String key = UUID.randomUUID().toString().substring(0, 6);
        LocalDate now = LocalDate.now();
        messageSendRequest.setUserKey(now.toString().substring(2).replace("-", "") + key);

        if (messageSendRequest.getSendType() == null || messageSendRequest.getSendType().isEmpty()) {
            if (messageSendRequest.getContents().getBytes(StandardCharsets.UTF_8).length < 91) {
                messageSendRequest.setSendType("SMS");
                return this.messageSend(messageSendRequest, url);
            } else {
                messageSendRequest.setSendType("LMS");
                url = lms;
                return this.messageSend(messageSendRequest, url);
            }
        } else {
            switch (messageSendRequest.getSendType()) {
                case "SMS":
                    return this.messageSend(messageSendRequest, url);
                case "LMS":
                    url = lms;
                    return this.messageSend(messageSendRequest, url);
                case "MMS":
                    url = mms;
                    return this.messageSend(messageSendRequest, url);
                case "KAKAO":
                    url = kakao;
                    return this.messageSend(messageSendRequest, url);
                default:
                    return null;
            }
        }
    }

    public MessageSendResponse messageSend(MessageSendRequest messageSendRequest, String url) throws IOException, InterruptedException {
        MessageSendResponse messageSendResponse = new MessageSendResponse();
        SejongMessageSendResponse sejongMessageSendResponse = null;

        try {
            ObjectMapper objectMapper = new ObjectMapper();
            Map<String, String> body = this.prepareRequestBody(messageSendRequest, objectMapper);
            String boundary = "----WebKitFormBoundary" + System.currentTimeMillis();

            // Multipart body 생성
            HttpRequest.BodyPublisher bodyPublisher = createMultipartBody(body, boundary);

            HttpRequest request = HttpRequest.newBuilder()
                    .uri(URI.create(url))
                    .header("Content-Type", "multipart/form-data; boundary=" + boundary)
                    .header("sejongApiKey", this.apiKey)
                    .POST(bodyPublisher)
                    .build();

            HttpResponse<String> response = this.httpClient.send(request, HttpResponse.BodyHandlers.ofString());
            System.out.println("responseCode : " + response.statusCode());
            System.out.println("responseBody : " + response.body());
            System.out.println("responseFull : " + response);

            if (response.statusCode() == 200) {
                sejongMessageSendResponse = objectMapper.readValue(response.body(), SejongMessageSendResponse.class);
            } else {
                messageSendResponse.setCode("500");
                messageSendResponse.setMessage("서버 통신 오류");
            }
        } catch (Exception e) {
            e.printStackTrace();
            messageSendResponse.setCode("500");
            messageSendResponse.setMessage("서버 통신 오류");
        }

        if (this.messageSendServiceDsl == null) {
            System.out.println("messageSendServiceDsl is null");
            throw new IllegalStateException("messageSendServiceDsl is not initialized.");
        }

        if (sejongMessageSendResponse == null) {
            System.out.println("sejongMessageSendResponse is null");
        }

        if (messageSendRequest == null) {
            System.out.println("messageSendRequest is null");
        }

        if (messageSendResponse == null) {
            System.out.println("messageSendResponse is null");
        }

        messageSendResponse = this.responseChange(sejongMessageSendResponse, messageSendRequest);
        System.out.println("sejongMessageSendResponse : " + sejongMessageSendResponse);
        System.out.println("messageSendRequest : " + messageSendRequest);
        System.out.println("messageSendResponse : " + messageSendResponse);
        this.messageSendServiceDsl.insertSendLog(sejongMessageSendResponse, messageSendRequest, messageSendResponse);
        System.out.println("11111111111111111111");
        if ("KAKAO".equals(messageSendRequest.getSendType()) && "Y".equals(messageSendRequest.getOptionYn()) &&
                !Objects.equals(sejongMessageSendResponse.getCode(), "200")) {
            messageSendRequest.setSendType("");
            messageSendResponse = this.messageDiv(messageSendRequest);
        }
        System.out.println("22222222222222222222");
        return messageSendResponse;
    }

    private Map<String, String> prepareRequestBody(MessageSendRequest messageSendRequest, ObjectMapper objectMapper) {
        switch (messageSendRequest.getSendType()) {
            case "SMS":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongSmsSendRequest.class), new TypeReference<>() {});
            case "LMS":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongLmsSendRequest.class), new TypeReference<>() {});
            case "MMS":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongMmsSendRequest.class), new TypeReference<>() {});
            case "KAKAO":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongKakaoSendRequest.class), new TypeReference<>() {});
            default:
                throw new IllegalArgumentException("Unsupported send type");
        }
    }

    private HttpRequest.BodyPublisher createMultipartBody(Map<String, String> body, String boundary) {
        StringBuilder multipartBody = new StringBuilder();

        for (Map.Entry<String, String> entry : body.entrySet()) {
            multipartBody.append("--").append(boundary).append("\r\n");
            multipartBody.append("Content-Disposition: form-data; name=\"").append(entry.getKey()).append("\"\r\n\r\n");
            multipartBody.append(entry.getValue()).append("\r\n");
        }

        // 마지막 바운더리 추가
        multipartBody.append("--").append(boundary).append("--\r\n");

        return HttpRequest.BodyPublishers.ofString(multipartBody.toString());
    }

    public MessageSendResponse responseChange(SejongMessageSendResponse response, MessageSendRequest request) {
        MessageSendResponse messageSendResponse = new MessageSendResponse();
        messageSendResponse.setSendType(request.getSendType());
        messageSendResponse.setSendCode(response != null ? response.getSendCode() : null);

        if (response == null || response.getCode() == null) {
            messageSendResponse.setCode("500");
            messageSendResponse.setMessage("서버 오류");
            return messageSendResponse;
        }

        int code = Integer.parseInt(response.getCode());
        if (code == 200) {
            messageSendResponse.setCode("200");
            messageSendResponse.setMessage("메세지 전송 요청 성공");
        } else if (code >= 300 && code < 400) {
            messageSendResponse.setCode("300");
            messageSendResponse.setMessage("Parameter 확인");
        } else if (code >= 400 && code < 500) {
            messageSendResponse.setCode("400");
            messageSendResponse.setMessage("서버 통신 실패");
        } else if (code >= 500) {
            messageSendResponse.setCode("600");
            messageSendResponse.setMessage("메세지 전송 오류");
        }

        return messageSendResponse;
    }
}